package com.gic.haoban.manage.web.exception;

import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.web.controller.WebBaseController;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

/**
 * Created 2018/7/24.
 *
 * @author hua
 */
@ControllerAdvice
public class GlobalExceptionHandler extends WebBaseController {
    private static final Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    /**
     * RuntimeException 类型异常告警
     *
     * @param response
     * @param ex
     * @return
     */
    @ResponseBody
    @ExceptionHandler(RuntimeException.class)
    public HaobanResponse controllerException(HttpServletResponse response, RuntimeException ex) {
        logger.error("Exception:{}", ex.getMessage(), ex);
        HaoBanErrCode errCode = HaoBanErrCode.ERR_3;
        StringBuilder sb = new StringBuilder();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (PrintWriter printWriter = new PrintWriter(baos)) {
            ex.printStackTrace(printWriter);
        }
        try {
            sb.append(baos.toString());
        } catch (Exception ignored) {
        }
        if (sb.length() == 0) {
            sb.append(ex.getMessage());
        }
        // 输出详细错误信息，便于调试
        return resultResponse(errCode, null, sb.toString());
    }

    /**
     * valid 异常告警
     *
     * @param response
     * @param ex
     * @return
     */
    @ResponseBody
    @ExceptionHandler(BindException.class)
    public HaobanResponse bindException(HttpServletResponse response, BindException ex) {
        logger.error("Exception:{}", ex.getMessage(), ex);
        HaoBanErrCode errCode = HaoBanErrCode.ERR_5;
        BindingResult bindingResult = ex.getBindingResult();
        FieldError fieldError = bindingResult.getFieldError();
        return resultResponse(errCode, null, fieldError.getField() + fieldError.getDefaultMessage());
    }

    /**
     * valid 异常告警 @ReuqestBody
     *
     * @param response
     * @param ex
     * @return
     */
    @ResponseBody
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public HaobanResponse methodArgumentNotValidException(HttpServletResponse response, MethodArgumentNotValidException ex) {
        logger.error("Exception:{}", ex.getMessage(), ex);
        HaoBanErrCode errCode = HaoBanErrCode.ERR_5;
        BindingResult bindingResult = ex.getBindingResult();
        FieldError fieldError = bindingResult.getFieldError();
        return resultResponse(errCode, null, fieldError.getField() + fieldError.getDefaultMessage());
    }

    @ResponseBody
    @ExceptionHandler(Exception.class)
    public HaobanResponse globalException(HttpServletResponse response, Exception ex) {
        logger.error("Exception:{}", ex.getMessage(), ex);
        HaoBanErrCode errCode = HaoBanErrCode.ERR_3;
        StringBuilder sb = new StringBuilder();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (PrintWriter printWriter = new PrintWriter(baos)) {
            ex.printStackTrace(printWriter);
        }
        try {
            sb.append(baos.toString());
        } catch (Exception ignored) {
        }
        if (sb.length() == 0) {
            sb.append(ex.getMessage());
        }
        // 输出详细错误信息，便于调试
        return resultResponse(errCode, null, sb.toString());
    }


}
